-- 
-- FarmingTablet - Trigger for placeable or installable Loadingstation
-- 
-- @Interface: 1.4.4.0 1.4.4RC8
-- @Author: kevink98 
-- @Date: 10.06.2017
-- @Version: 1.0.0
-- 
-- @Support: http://ls-modcompany.com
-- @mCompanyInfo: http://mcompany-info.de/
-- 

FarmingTabletTrigger = {};
FarmingTabletTrigger_directory = g_currentModDirectory;
FarmingTabletTrigger_directoryMods = g_modsDirectory;

FarmingTabletTrigger.version = "1.0.0";
print("   [FarmingTablet - Trigger] load " .. tostring(FarmingTabletTrigger.version));

local FarmingTabletTrigger_mt = nil;

local n, _ = string.find(FarmingTabletTrigger_directory, "FarmingTablet");
if n then
	FarmingTabletTrigger_mt = Class(FarmingTabletTrigger, Placeable);
else
	FarmingTabletTrigger.isObject = true;
	FarmingTabletTrigger_mt = Class(FarmingTabletTrigger, Object);
end;

InitObjectClass(FarmingTabletTrigger, "FarmingTabletTrigger");

function FarmingTabletTrigger.onCreate(id)
	local trigger = FarmingTabletTrigger:new(g_server ~= nil, g_client ~= nil)
	if trigger ~= nil then
		g_currentMission:addOnCreateLoadedObject(trigger)
		trigger:load(id)
		trigger:register(true)
	else
		trigger:delete();
	end;
end;

function FarmingTabletTrigger:new(isServer, isClient, cMt)
	local mt = customMt;
	if mt == nil then
		mt = FarmingTabletTrigger_mt;
	end;
	
	local self = {};
	if FarmingTabletTrigger.isObject then
		self = Object:new(isServer, isClient, FarmingTabletTrigger_mt);
	else
		self = Placeable:new(isServer, isClient, FarmingTabletTrigger_mt);
		registerObjectClassName(self, "FarmingTabletTrigger");
	end;
	
	return self;
end;

function FarmingTabletTrigger:load(id, x,y,z, rx,ry,rz, initRandom)
	if self.isObject then
		if not self:finalizePlacement() then
			return false 
		end;
	else
		if not FarmingTabletTrigger:superClass().load(self, id, x,y,z, rx,ry,rz, initRandom) then
			return false;
		end;
		
		return true;
	end;
	
	return true;
end;

function FarmingTabletTrigger:finalizePlacement()
	if not self.isObject then
		FarmingTabletTrigger:superClass().finalizePlacement(self)		
	end	
	
	local a = Utils.indexToObject(self.nodeId, "8");
	self.animatedObjects = {};
	for i=1, getNumOfChildren(a) do
		local an = getChildAt(a, i-1);
		local animatedObject = AnimatedObject:new(self.isServer, self.isClient)
		animatedObject.baseDirectory = self.baseDirectory
		animatedObject.customEnvironment = self.customEnvironment
		if animatedObject:load(an) then
			animatedObject:register(true)
			table.insert(self.animatedObjects, animatedObject)
		end
	end;
	
	self.triggerId = getChild(self.nodeId, "PlayerTrigger1");
	self.tablet = getChild(self.nodeId, "Tablet");
	self.screen_background = getChild(self.nodeId, "background");
	self.screen_startLoading = getChild(self.nodeId, "startloading");
	self.screen_endLoading = getChild(self.nodeId, "loadingend");
	
	local loadings = getChild(self.nodeId, "loading");
	self.screen_loading_1 = getChild(loadings, "loading1");
	self.screen_loading_2 = getChild(loadings, "loading2");
	self.screen_loading_3 = getChild(loadings, "loading3");
	self.screen_loading_4 = getChild(loadings, "loading4");
		
	self.isLoading = false;
	self.isInStation = false;
	self.playerInTrigger = false;
	addTrigger(self.triggerId, "triggerCallback", self);
	
	self.appearsOnPDA = Utils.getNoNil(getUserAttribute(self.nodeId, "appearsOnPDA"), true);
	self.title = g_i18n:getText(Utils.getNoNil(getUserAttribute(self.nodeId, "titel"), "FarmingTablet_LoadingTitel"));
	
	if self.appearsOnPDA then
		local x, _, z = getWorldTranslation(self.nodeId);
		self.hotspot = g_currentMission.ingameMap:createMapHotspot("farmingTabletTrigger", self.title, FarmingTabletTrigger_directory .. "Ladestation_pda.dds", nil, nil, x, z, nil, nil, false, false, true, nil, nil, MapHotspot.CATEGORY_DEFAULT);
	end;
	
	if self.isObject then
		g_currentMission:addNodeObject(self.nodeId, self)
	end;
	
	return true;
end;

function FarmingTabletTrigger:readStream(streamId, connection)
    FarmingTabletTrigger:superClass().readStream(self, streamId, connection);
    if connection:getIsServer() then
        for i=1, #self.animatedObjects do
            local animatedObject = self.animatedObjects[i]
            local animatedObjectId = readNetworkNodeObjectId(streamId);
            animatedObject:readStream(streamId, connection);
            g_client:finishRegisterObject(animatedObject, animatedObjectId);
        end
    end
end

function FarmingTabletTrigger:writeStream(streamId, connection)
    FarmingTabletTrigger:superClass().writeStream(self, streamId, connection);
    if not connection:getIsServer() then
        for i=1, #self.animatedObjects do
            local animatedObject = self.animatedObjects[i]
            writeNetworkNodeObjectId(streamId, animatedObject.id)
            animatedObject:writeStream(streamId, connection)
            g_server:registerObjectInStream(connection, animatedObject)
        end
    end
end

function FarmingTabletTrigger:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    if not FarmingTabletTrigger:superClass().loadFromAttributesAndNodes(self, xmlFile, key, resetVehicles) then
        return false;
    end;

    for i=1, #self.animatedObjects do
        local animatedObject = self.animatedObjects[i]
        if hasXMLProperty(xmlFile, key..".animatedObject("..(i-1)..")") then
            animatedObject:loadFromAttributesAndNodes(xmlFile, key..".animatedObject("..(i-1)..")")
        end
    end

    return true;
end;

function FarmingTabletTrigger:getSaveAttributesAndNodes(nodeIdent)
    local attributes, nodes = FarmingTabletTrigger:superClass().getSaveAttributesAndNodes(self, nodeIdent);

    for i=1, #self.animatedObjects do
        local animatedObject = self.animatedObjects[i]
        local a, _ = animatedObject:getSaveAttributesAndNodes(nodeIdent .. "    ")
        if i > 1 then
            nodes = nodes.."\n";
        end;
        nodes = nodes..nodeIdent.."<animatedObject".." "..a.. "/>"
    end

    return attributes, nodes;
end;

function FarmingTabletTrigger:update(dt)	
	if self.playerInTrigger then
		if self.isInStation then
			g_currentMission:addHelpButtonText(g_i18n:getText("input_FTturnOn2"), InputBinding.FTturnOn, nil, GS_PRIO_VERY_LOW)
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("input_FTturnOn"), InputBinding.FTturnOn, nil, GS_PRIO_VERY_LOW)
		end;
		
		if InputBinding.hasEvent(InputBinding.FTturnOn) then
			self.isInStation = not self.isInStation;
			g_farmingTablet.setLoadingStation(self.isInStation);
			self:reset();
			self:setTabletStation(self.isInStation);
			if not self.isInStation then
				self.isLoading = false;
				g_farmingTablet.setLoading(self.isLoading);
			end;
		elseif InputBinding.hasEvent(InputBinding.FTloadTab) and self.isInStation then
			self.isLoading = not self.isLoading;
			g_farmingTablet.setLoading(self.isLoading);
			self:reset();
			if not self.isLoading then
				setVisibility(self.tablet, true);
				setVisibility(self.screen_background, true);
				setVisibility(self.screen_startLoading, true);
			end;
		end;
	end;
	
	if self.isLoading then
		local lvl, capacity = g_farmingTablet.getAkkuState();
		setVisibility(self.tablet, true);
		setVisibility(self.screen_background, true);
		setVisibility(self.screen_startLoading, false);
		setVisibility(self.screen_endLoading, false);
		setVisibility(self.screen_loading_1, false);
		setVisibility(self.screen_loading_2, false);
		setVisibility(self.screen_loading_3, false);
		setVisibility(self.screen_loading_4, false);
		
		local fLvl = 100 / capacity * lvl;
		
		if  fLvl <= 25 then
			setVisibility(self.screen_loading_1, true);
		elseif fLvl <= 50 then
			setVisibility(self.screen_loading_2, true);
		elseif fLvl <= 75 then
			setVisibility(self.screen_loading_3, true);
		elseif fLvl < 100 then
			setVisibility(self.screen_loading_4, true);
		elseif fLvl == 100 then
			setVisibility(self.screen_endLoading, true);
			self.isLoading = not self.isLoading;
			g_farmingTablet.setLoading(false);
		end;		
	end;
end;

function FarmingTabletTrigger:setTabletStation(s)
	if s then
		setVisibility(self.tablet, s);
		setVisibility(self.screen_background, s);
		setVisibility(self.screen_startLoading, s);
	else
		self:reset();
	end;	
end;

function FarmingTabletTrigger:reset()
	setVisibility(self.screen_startLoading, false);
	setVisibility(self.screen_endLoading, false);
	setVisibility(self.screen_loading_1, false);
	setVisibility(self.screen_loading_2, false);
	setVisibility(self.screen_loading_3, false);
	setVisibility(self.screen_loading_4, false);
	setVisibility(self.screen_background, false);
	setVisibility(self.tablet, false);
end;

function FarmingTabletTrigger:deleteMap()
	self:delete();
end;

function FarmingTabletTrigger:delete()
	if self.hotspot ~= nil then
		g_currentMission.ingameMap:deleteMapHotspot(self.hotspot);
	end;
	if self.triggerId ~= nil then
		removeTrigger(self.triggerId);
	end;
	if not self.isObject then 
		if self.animatedObjects then
			for i=1, #self.animatedObjects do
				local animatedObject = self.animatedObjects[i]
				animatedObject:unregister(true)
				animatedObject:delete()
			end
		end;
		self:superClass().delete(self) 
	end;
end;

function FarmingTabletTrigger:triggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	if onEnter and not self.isLoading then
		if g_farmingTablet ~= nil and g_farmingTablet.setLoading ~= nil then
			self.playerInTrigger = true;
		end;
	elseif onLeave then
		if g_farmingTablet ~= nil and g_farmingTablet.setLoading ~= nil then
			self.playerInTrigger = false;
		end;
	end;
end;

if FarmingTabletTrigger.isObject then
	g_onCreateUtil.addOnCreateFunction("FarmingTabletTrigger", FarmingTabletTrigger.onCreate);
else
	registerPlaceableType("FarmingTabletTrigger", FarmingTabletTrigger);
end;