-- 
-- FarmingTablet - Utils
-- 
-- @Interface: 1.4.4.0 1.4.4RC8
-- @Author: kevink98 
-- @Date: 25.09.2017
-- @Version: 1.1.0
-- 
-- @Support: http://ls-modcompany.com
-- @mCompanyInfo: http://mcompany-info.de/
-- 

FarmingTabletUtils = {};
FarmingTabletUtils. version = "1.0.0"
g_farmingTablet.utils = FarmingTabletUtils;

FarmingTabletUtils.text = {};

function FarmingTabletUtils.drawTextButton(button, text, h, f, c1, c2, bold)
	if button == nil then return end;
	if text == nil then text = "" end;
	if h == nil then h = 0.028 end;
	if f == nil then f = 2 end;
	if c1 == nil then c1 = {0.408,0.412,0.408,1} end;
	if c2 == nil then c2 = {0.3843,0.69,0.1176,1} end;
	
	if button.hover or button.ActivateState then
		setTextColor(unpack(c1));
	else
		setTextColor(unpack(c2));
	end;
	setTextBold(bold or false);
	local x = button.posXL + ((button.posXR-button.posXL)/2)-(getTextWidth(h ,text) /2);
	local y = button.posYU +(getTextHeight(h , text) /f);
	renderText(x,y, h, text);
	
	setTextBold(false);
	setTextColor(0,0,0,1);
end;

function FarmingTabletUtils.loadLanguageText(nameApp, xmlFileName)
	FarmingTabletUtils.text[nameApp] = {};
	local xml = loadXMLFile("languageXML", xmlFileName, "languageXML");
	local i = 0;	
	while true do
		local key = string.format("languageXML.text(%d)", i);
		if not hasXMLProperty(xml, key) then			
			break;
		end;
		local name = getXMLString(xml, key .. "#name");
		local text = Utils.getNoNil(getXMLString(xml, key .. "#" .. g_languageShort),getXMLString(xml, key .. "#en"))
		FarmingTabletUtils.text[nameApp][name] = text;
		i = i + 1;
	end; 
end;

function FarmingTabletUtils.getTextByAppName(nameApp, name, getReturn)
	if FarmingTabletUtils.text[nameApp] ~= nil and FarmingTabletUtils.text[nameApp][name] ~= nil then
		return FarmingTabletUtils.text[nameApp][name], true;
	elseif getReturn == nil or getReturn then
		return "'" .. name .. "' not found in languageXML (" .. nameApp .. ")", false;
	end;
end;

function FarmingTabletUtils.addNew_Category_Window(app, appName, name, category, showDetails,isClick,isHover,drawFirst)
	g_farmingTablet.button.newCategory(category);	
	g_farmingTablet.newWindow(appName, name,category,showDetails,isClick,isHover);
	local update = function(...) 
		if app["update_" .. name] ~= nil then
			return app["update_" .. name](...);
		else
			return function() end;
		end;
	end;
	local draw = function(...) 
		if app["draw_" .. name] ~= nil then
			return app["draw_" .. name](...);
		else
			return function() end;
		end;
	end;	
	local key = function(...) 
		if app["key_" .. name] ~= nil then
			return app["key_" .. name](...);
		else
			return function() end;
		end;	
	end;
	g_farmingTablet.setFuncWindows(name,update,draw,key,drawFirst)
end;

function FarmingTabletUtils.loadOverlay(appName, dir, xmlFileName)
	local i = 0;
	local xml = loadXMLFile("overlays", dir .. xmlFileName, "overlays");
	local overlays = {};
	while true do
		local key = string.format("overlays.overlay(%d)",i);
		if not hasXMLProperty(xml, key) then
			break;
		end;
		local overlay = {};
		overlay.name = getXMLString(xml, key .. "#name");
		overlay.modus = getXMLString(xml, key .. "#modus");
		local textColor = getXMLString(xml, key .. "#textColor");
		local file = getXMLString(xml, key .. "#file");
		overlay.text = g_farmingTablet.utils.getTextByAppName(appName, overlay.name);
		if textColor ~= nil then
			overlay.colors = Utils.splitString(" ", textColor);
		end;
		overlay.id = createImageOverlay(dir .. file);
		table.insert(overlays, overlay);
		i = i + 1;
	end;
	g_farmingTablet.button.addOverlays(overlays)
end;

function FarmingTabletUtils.loadButtons(appName, xmlFileName, app, appNameButtonText)
	local i = 0;
	local xml = loadXMLFile("buttons", xmlFileName, "buttons");
	local buttons = {};
	while true do
		local key = string.format("buttons.button(%d)",i);
		if not hasXMLProperty(xml, key) then
			break;
		end;
		
		local name = getXMLString(xml, key .. "#name");
		local category = getXMLString(xml, key .. "#category");
		local pos = Utils.splitString(" ",getXMLString(xml, key .. "#pos"));
		local scale = Utils.splitString(" ",getXMLString(xml, key .. "#scale"));
		local activateState = getXMLBool(xml, key .. "#activateState");
		local activated = getXMLBool(xml, key .. "#activated");		
		local visible = getXMLBool(xml, key .. "#visible");
		local useOwnFunc = getXMLBool(xml, key .. "#useOwnFunc");
		local hover = getXMLString(xml, key .. "#hover");
		local default = getXMLString(xml, key .. "#default");
		local active = getXMLString(xml, key .. "#active");		
		local para = getXMLInt(xml, key .. "#parameter");
		local funcName = getXMLString(xml, key .. "#funcName");
		local textHeight = getXMLFloat(xml, key .. "#textHeight");
		local c1 = getXMLString(xml, key .. "#colorTextHover");
		local c2 = getXMLString(xml, key .. "#colorText");
		local d = getXMLFloat(xml, key .. "#distance");
		local x,y = unpack(pos);
		local xS,yS = unpack(scale);
		
		local c1_1 = nil;
		local c2_1 = nil;
		if c1 ~= nil and c2 ~= nil then
			local r,g,b,a = unpack(Utils.splitString(" ",c1));
			local r1,g1,b1,a1 = unpack(Utils.splitString(" ",c2));
			c1_1 = {tonumber(r),tonumber(g),tonumber(b),tonumber(a)}
			c2_1 = {tonumber(r1),tonumber(g1),tonumber(b1),tonumber(a1)}
		end;
		g_farmingTablet.button.newButton(app, appName, name,category,tonumber(x),tonumber(y),tonumber(xS),tonumber(yS),hover,default,active,useOwnFunc, visible, activateState, funcName, para, textHeight,c1_1,c2_1,d,activated, appNameButtonText)
		i = i + 1;
	end;	
	i = 0;
	while true do
		local key = string.format("buttons.slideBar(%d)",i);
		if not hasXMLProperty(xml, key) then
			break;
		end;
		
		local name = getXMLString(xml, key .. "#name");
		local category = getXMLString(xml, key .. "#category");
		local pos = Utils.splitString(" ",getXMLString(xml, key .. "#pos"));
		local scale = Utils.splitString(" ",getXMLString(xml, key .. "#scale"));
		local sliderSize = Utils.splitString(" ",getXMLString(xml, key .. "#sliderSize"));
		local values = Utils.splitString(" ",getXMLString(xml, key .. "#values"));
		local startValue =  getXMLString(xml, key .. "#startValue");
		local background = getXMLString(xml, key .. "#background");		
		local slider = getXMLString(xml, key .. "#slider");		
		local sliderHover = getXMLString(xml, key .. "#sliderHover");	
		local funcSet = getXMLString(xml, key .. "#funcNameSet");	
		local useSliderSize = Utils.getNoNil(getXMLBool(xml, key .. "#useSliderSize"), true);
		local x,y = unpack(pos);
		local xS,yS = unpack(scale);	
		local s1,s2 = unpack(sliderSize);	
		local vS,vE = unpack(values);	
		g_farmingTablet.button.newSlideBar(app, appName, name, category, tonumber(x),tonumber(y),tonumber(xS),tonumber(yS),tonumber(s1),tonumber(s2),tonumber(vS),tonumber(vE), background, slider, sliderHover, funcSet, useSliderSize);
		if startValue ~= nil then
			local s1,s2 = unpack(Utils.splitString(" ",startValue));	
			g_farmingTablet.button.getSlidebarByCategoryName(category, name).stateWidth = tonumber(s1);
			if s2 ~= nil then
				g_farmingTablet.button.getSlidebarByCategoryName(category, name).stateHeight = tonumber(s2);
			end;
		end;
		i = i + 1;
	end;
end;

function FarmingTabletUtils.getActiveTextColor()
	local r,g,b,a;
	if g_farmingTablet.settings.useOwnTextColor then
		r = FarmingTabletButton.getSlidebarByCategoryName("SETTINGS_PAGE2", "slideBar_settings_r").stateWidth;
		g = FarmingTabletButton.getSlidebarByCategoryName("SETTINGS_PAGE2", "slideBar_settings_g").stateWidth;
		b = FarmingTabletButton.getSlidebarByCategoryName("SETTINGS_PAGE2", "slideBar_settings_b").stateWidth;
		a = FarmingTabletButton.getSlidebarByCategoryName("SETTINGS_PAGE2", "slideBar_settings_a").stateWidth;
	else
		r,g,b = unpack(FarmingTablet.button.getOverlayByModus_Id("home", g_farmingTablet.activeOverlay).colors)
		a = 1;
	end;	
	return tonumber(r), tonumber(g), tonumber(b), tonumber(a);
end;

function FarmingTabletUtils.getActiveAppTextColor()
	local r,g,b,a;
	if FarmingTablet.activeAppName ~= "" then		
		if g_farmingTablet.apps[FarmingTablet.activeAppName].settings.useOwnTextColor then
			r = g_farmingTablet.apps[FarmingTablet.activeAppName].settings.r;
			g = g_farmingTablet.apps[FarmingTablet.activeAppName].settings.g;
			b = g_farmingTablet.apps[FarmingTablet.activeAppName].settings.b;
			a = g_farmingTablet.apps[FarmingTablet.activeAppName].settings.a;
		else
			r,g,b = unpack(FarmingTablet.button.getOverlayByModus_Id("app", g_farmingTablet.apps[FarmingTablet.activeAppName].backgroundActiveIndex).colors)
			a = 1;
		end;	
		return tonumber(r), tonumber(g), tonumber(b), tonumber(a);
	else
		return 1,1,1,1;
	end;
end;




